//libraries
#include <DRV8835MotorShield.h>
#include <EEPROM.h>
#include "SerialCommand.h"
#include "EEPROMAnything.h"
#include "Wire.h"
#include <sensorbar.h>

//parameters
  //communication
  #define SerialPort Serial1
  SerialCommand sCmd(SerialPort);
  //motors
  DRV8835MotorShield motors;
  //sensors
  const uint8_t SX1509_ADDRESS = 0x3E;
  SensorBar mySensorBar(SX1509_ADDRESS);

//variables
  int positie, vorigePositie, verschilPositie, m1Speed, m2Speed;
  //pins
  int interruptPin = 2;
  //timeVars  
  unsigned long time, prevTime, delta, calculationTime, cycleTime;
  //declareer datatype Parameters van struct paramsDataType
  struct paramsDataType{
    bool start;
    int speed;
    float kp;
    float ki;
    float kd;
    unsigned long cycleTime;
    int wit [6];
    int zwart [6];
    };
  typedef struct paramsDataType Parameters;
  //aanmaken variabele params van dataType Parameters
  Parameters params;

void setup(){  
  //pinMode(interruptPin, INPUT_PULLUP);
  //attachInterrupt(digitalPinToInterrupt(interruptPin), onInterrupt, RISING);
  
  time = micros();
  prevTime = time;
  
  SerialPort.begin(9600); 
  EEPROM_readAnything(0, params);
  params.start = false;

  sCmd.addCommand("start", onStart);
  sCmd.addCommand("stop", onStop);
  sCmd.addCommand("set", onSet);
  sCmd.addCommand("debug", onDebug);
  sCmd.addCommand("reset", onReset);
  sCmd.setDefaultHandler(onUnknownCommand);

  //Instellingen Sensorbar
    //instelling enkel sensoren laten uitlezen als arduino uitleest, anders: "mySensorBar.clearBarStrobe();"
    mySensorBar.setBarStrobe();
    //Instellen voor zwarte lijn op wit papier
    mySensorBar.clearInvertBits();
    //Start uitlezen
    uint8_t returnStatus = mySensorBar.begin();
  
  //Test sensorbar
  if(returnStatus){
    Serial.println("sx1509 IC communication OK");}
  else{
    Serial.println("sx1509 IC communication FAILED!");
    while(1){Serial.println("sx1509 IC communication FAILED!");}}
  
  SerialPort.println("ready");
}

void loop(){  
  sCmd.readSerial();  
  
  time = micros();
  if (time > prevTime){
    delta = time - prevTime;}
  else{
    delta = 4294967295 - prevTime + time + 1;}
    
  if (delta > params.cycleTime){
    prevTime = time;
      
    //Aansturing motors
      positie = mySensorBar.getPosition();
      verschilPositie = positie - vorigePositie;
      if (params.start==true){
        m1Speed = params.speed - (params.kp * positie + params.ki * vorigePositie - params.kd * verschilPositie);
        m2Speed = params.speed + (params.kp * positie + params.ki * vorigePositie - params.kd * verschilPositie);
        if(m1Speed < 0){
          m1Speed = m1Speed/2;
          }
        if(m2Speed < 0){
          m2Speed = m2Speed/2;
        }
        motors.setM1Speed(m1Speed);
        motors.setM2Speed(m2Speed);}
      else{
      motors.setM1Speed(0);
      motors.setM2Speed(0);}
      vorigePositie = positie;
    }
    
    unsigned long difference = micros() - time;
    
    if (difference > calculationTime){ 
      calculationTime = difference;}
}

void onUnknownCommand(char *command){
  SerialPort.print("unknown command: \"");
  SerialPort.print(command);
  SerialPort.println("\"");
}

void onSet(){
  char* parameter = sCmd.next();
  char* value = sCmd.next();
  
  if (strcmp(parameter, "speed") == 0) params.speed = atof(value);
  else if (strcmp(parameter, "kp") == 0) params.kp = atof(value);
  else if (strcmp(parameter, "ki") == 0) params.ki = atof(value);
  else if (strcmp(parameter, "kd") == 0) params.kd = atof(value);
  else if (strcmp(parameter, "cycle") == 0) params.cycleTime = atof(value);
  
  EEPROM_writeAnything(0, params);
}

void onStart(){
  params.start = true;
  SerialPort.print("vroem");
}

void onStop(){
  params.start = false;
  motors.setM1Speed(0);
  motors.setM2Speed(0);
}

void onDebug(){
  //parameters
    SerialPort.print("speed: ");
    SerialPort.println(params.speed);
    SerialPort.print("kp: ");
    SerialPort.println(params.kp);   
    SerialPort.print("ki: ");
    SerialPort.println(params.ki);
    SerialPort.print("kd: ");
    SerialPort.println(params.kd);
  //cycle times
    SerialPort.print("cycle time: ");
    SerialPort.println(params.cycleTime);
    SerialPort.print("calculation time: ");
    SerialPort.println(calculationTime);
    calculationTime = 0;  //reset calculation time
  //running
    SerialPort.print("running: ");
    SerialPort.println(params.start); 
}

void onReset(){
  SerialPort.print("resetting parameters... ");
  EEPROM_resetAnything(0, params);
  EEPROM_readAnything(0, params);  
  SerialPort.println("done");
}

void onInterrupt(){
  params.start = not params.start;
}
